@@ -4,22 +4,18 @@
 
     Utility functions common to the C and C++ domains.
 
-    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
-import warnings
 from copy import deepcopy
-from typing import (
-    Any, Callable, List, Match, Optional, Pattern, Tuple, Union
-)
+from typing import Any, Callable, List, Match, Optional, Pattern, Tuple, Union
 
 from docutils import nodes
 from docutils.nodes import TextElement
 
 from sphinx.config import Config
-from sphinx.deprecation import RemovedInSphinx40Warning
 from sphinx.util import logging
 
 logger = logging.getLogger(__name__)
@@ -87,12 +83,7 @@ def verify_description_mode(mode: str) -> None:
 
 class NoOldIdError(Exception):
     # Used to avoid implementing unneeded id generation for old id schemes.
-    @property
-    def description(self) -> str:
-        warnings.warn('%s.description is deprecated. '
-                      'Coerce the instance to a string instead.' % self.__class__.__name__,
-                      RemovedInSphinx40Warning, stacklevel=2)
-        return str(self)
+    pass
 
 
 class ASTBaseBase:
@@ -107,7 +98,7 @@ class ASTBaseBase:
             return False
         return True
 
-    __hash__ = None  # type: Callable[[], int]
+    __hash__: Callable[[], int] = None
 
     def clone(self) -> Any:
         return deepcopy(self)
@@ -215,21 +206,11 @@ class ASTBaseParenExprList(ASTBaseBase):
 ################################################################################
 
 class UnsupportedMultiCharacterCharLiteral(Exception):
-    @property
-    def decoded(self) -> str:
-        warnings.warn('%s.decoded is deprecated. '
-                      'Coerce the instance to a string instead.' % self.__class__.__name__,
-                      RemovedInSphinx40Warning, stacklevel=2)
-        return str(self)
+    pass
 
 
 class DefinitionError(Exception):
-    @property
-    def description(self) -> str:
-        warnings.warn('%s.description is deprecated. '
-                      'Coerce the instance to a string instead.' % self.__class__.__name__,
-                      RemovedInSphinx40Warning, stacklevel=2)
-        return str(self)
+    pass
 
 
 class BaseParser:
@@ -242,9 +223,9 @@ class BaseParser:
 
         self.pos = 0
         self.end = len(self.definition)
-        self.last_match = None  # type: Match
-        self._previous_state = (0, None)  # type: Tuple[int, Match]
-        self.otherErrors = []  # type: List[DefinitionError]
+        self.last_match: Match = None
+        self._previous_state: Tuple[int, Match] = (0, None)
+        self.otherErrors: List[DefinitionError] = []
 
         # in our tests the following is set to False to capture bad parsing
         self.allowFallbackExpressionParsing = True
@@ -375,7 +356,7 @@ class BaseParser:
         # TODO: add handling of string literals and similar
         brackets = {'(': ')', '[': ']', '{': '}'}
         startPos = self.pos
-        symbols = []  # type: List[str]
+        symbols: List[str] = []
         while not self.eof:
             if len(symbols) == 0 and self.current_char in end:
                 break
@@ -391,7 +372,7 @@ class BaseParser:
                       % startPos)
         return self.definition[startPos:self.pos]
 
-    def _parse_attribute(self) -> ASTAttribute:
+    def _parse_attribute(self) -> Optional[ASTAttribute]:
         self.skip_ws()
         # try C++11 style
         startPos = self.pos
